// JavaTalkClientGfx.java
//
// CS549 Spring 96, Oregon State University
// 
// Jean-Guy Speton, Maharajapuram Balakrishnan
//
// Graphics canvas window.

import java.awt.*;
import java.util.Vector;

/**
 * @author M. Balakrishnan
 * @author J. Speton
 * @see JavaTalkClient
 *
 * The JavaTalkClientGfx class is the window and containing GUI elements
 * which correspond the the JavaTalk graphical editor.
 */
public class JavaTalkClientGfx extends Frame {

  private JavaTalkClient client;

  // Drawing mode enumerations.
  public final static int Draw          = 0;
  public final static int Line          = 1;
  public final static int Rect          = 2;
  public final static int FillRect      = 3;
  public final static int RoundRect     = 4;
  public final static int FillRoundRect = 5;
  public final static int Oval          = 6;
  public final static int FillOval      = 7;
  public final static int Arc           = 8;
  public final static int FillArc       = 9;

  // GUI elements...
  private GridBagLayout gfx_layout;
  private Button gfx_clear;
  private Choice gfx_mode, gfx_color;
  private GfxCanvas gfx_canvas;

  private int start_x = 0, start_y = 0;
  private int end_x = 0, end_y = 0;
  private int prev_x = 0, prev_y = 0;

  private Color current_color = Color.black; // default color black
  private int draw_mode = Draw; //default mode is free draw

  Vector figures = new Vector(128, 128);
  
  public JavaTalkClientGfx(JavaTalkClient parent)
  {
    super("JavaTalk Graphics Canvas");
    setResizable(false);

    client = parent;

    gfx_layout = new GridBagLayout();
    this.setLayout(gfx_layout);
    gfx_clear = new Button("Clear");
    client.constrain(this, gfx_clear, 0, 0, 1, 1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER, 0.0, 0.0, 2, 2, 2, 2);
    client.constrain(this, new Label("  Drawing Mode:"), 1, 0, 1, 1,
	      GridBagConstraints.NONE, GridBagConstraints.EAST,
	      0.0, 0.0, 2, 2, 2, 0);
    gfx_mode = new Choice();
    gfx_mode.addItem("Draw");
    gfx_mode.addItem("Line");
    gfx_mode.addItem("Rect");
    gfx_mode.addItem("FillRect");
    gfx_mode.addItem("RoundRect");
    gfx_mode.addItem("FillRoundRect");
    gfx_mode.addItem("Oval");
    gfx_mode.addItem("FillOval");

    client.constrain(this, gfx_mode, 2, 0, 1, 1, GridBagConstraints.HORIZONTAL,
		     GridBagConstraints.CENTER, 0.0, 0.0, 2, 0, 2, 2);
    client.constrain(this, new Label("  Color:"), 3, 0, 1, 1,
		     GridBagConstraints.NONE, GridBagConstraints.EAST,
		     0.0, 0.0, 2, 2, 2, 0);
    gfx_color = new Choice();
    gfx_color.addItem("Black");
    gfx_color.addItem("Blue");
    gfx_color.addItem("Cyan");
    gfx_color.addItem("Gray");
    gfx_color.addItem("Green");
    gfx_color.addItem("Magenta");
    gfx_color.addItem("Orange");
    gfx_color.addItem("Pink");
    gfx_color.addItem("Red");
    gfx_color.addItem("Yellow");
    client.constrain(this, gfx_color, 4, 0, 1, 1, 
		     GridBagConstraints.HORIZONTAL,
		     GridBagConstraints.NORTH, 0.0, 0.0, 2, 2, 2, 2);
    gfx_canvas = new GfxCanvas(this);
    gfx_canvas.setBackground(Color.white);
    client.constrain(this, gfx_canvas, 0, 1, 5, 1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER, 1.0, 1.0, 2, 2, 2, 2);
    gfx_canvas.resize(400, 400);
  }

  public void paint(Graphics gfx)
  {
    Figure f;

    //    Graphics g = gfx_canvas.getGraphics();
    Graphics g = gfx;
    g.setPaintMode();

    for (int i = 0; i < figures.size(); i++) {
      f = (Figure)figures.elementAt(i);
     
      g.setColor(f.color);

      switch(f.draw_mode) {
      case Draw:
	// should never happen
	break;
      case Line:
	g.drawLine(f.start_x, f.start_y, f.end_x, f.end_y);
	break;
      case Rect:
	drawRect(g, f.start_x, f.start_y, f.end_x, f.end_y);
	break;
      case FillRect:
	drawFillRect(g, f.start_x, f.start_y, f.end_x, f.end_y);
	break;
      case RoundRect:
	drawRoundRect(g, f.start_x, f.start_y, f.end_x, f.end_y);
	break;
      case FillRoundRect:
	drawFillRoundRect(g, f.start_x, f.start_y, f.end_x, f.end_y);
	break;
      case Oval:
	drawOval(g, f.start_x, f.start_y, f.end_x, f.end_y);
	break;
      case FillOval:
	drawFillOval(g, f.start_x, f.start_y, f.end_x, f.end_y);
	break;
      default:
	System.err.println("JavaTalkClientGfx::paint(): unknown draw mode " +
			   f.draw_mode);
	break;
      }
    }
  }
	    
  // Change the cursor to cross hair if it enters graphics canvas
  public boolean mouseEnter(Event e, int x, int y)
  {
    if (e.target == gfx_canvas) {
      this.setCursor(Frame.CROSSHAIR_CURSOR);
      return true;
    }
    return false;
  }
  
  public boolean mouseExit(Event e, int x, int y)
  {
    if (e.target == gfx_canvas) {
      this.setCursor(Frame.DEFAULT_CURSOR);
      return true;
    }
    return false;
  } 
 
  // Select appropriate draw_mode and color at the click of a button
  public boolean action(Event e, Object arg)
  {
    if (e.target == gfx_clear) {
      // Draw full size rectangle in background color.
      Graphics g = gfx_canvas.getGraphics();
      g.setColor(gfx_canvas.getBackground());
      g.fillRect(0, 0, gfx_canvas.size().width, gfx_canvas.size().height);

      // Clear vector.
      if (figures.size() > 0)
	figures.removeAllElements();
    }

    else if (e.target == gfx_mode) {
      if (arg.equals("Draw"))
	draw_mode = Draw;
      else if (arg.equals("Line"))
	draw_mode = Line;
      else if (arg.equals("Rect"))
	draw_mode = Rect;
      else if (arg.equals("FillRect"))
	draw_mode = FillRect;
      else if (arg.equals("RoundRect"))
	draw_mode = RoundRect;
      else if (arg.equals("FillRoundRect"))
	draw_mode = FillRoundRect;
      else if (arg.equals("Oval"))
	draw_mode = Oval;
      else if (arg.equals("FillOval"))
	draw_mode = FillOval;
      else if (arg.equals("Arc"))
	draw_mode = Arc;
      else if (arg.equals("FillArc"))
	draw_mode = FillArc;

      return true;
    }
    
    else if (e.target == gfx_color) {
      if (arg.equals("Black"))
	current_color = Color.black;
      else if (arg.equals("Blue"))
	current_color = Color.blue;
      else if (arg.equals("Cyan"))
	current_color = Color.cyan;
      else if (arg.equals("Gray"))
	current_color = Color.gray;
      else if (arg.equals("Green"))
	current_color = Color.green;
      else if (arg.equals("Magenta"))
	current_color = Color.magenta;
      else if (arg.equals("Orange"))
	current_color = Color.orange;
      else if (arg.equals("Pink"))
	current_color = Color.pink;
      else if (arg.equals("Red"))
	current_color = Color.red;
      else if (arg.equals("Yellow"))
	current_color = Color.yellow;

      gfx_canvas.setForeground(current_color);

      return true;
    }

    return super.action(e, arg);
  }

  // Get the start coordinates for the element to be drawn.
  public boolean mouseDown(Event e, int x, int y)
  {
    if (e.target == gfx_canvas) {
      start_x = x - gfx_canvas.location().x;
      start_y = y - gfx_canvas.location().y;
      end_x = start_x;
      end_y = start_y;
      return true;
    }
    return false;
  }

  // Draw XORed shape as long as mouse button is held down.
  public boolean mouseDrag(Event e, int x, int y)
  {
    if (e.target != gfx_canvas)
      return false;

    Graphics g = gfx_canvas.getGraphics();
    
    prev_x = end_x;
    prev_y = end_y;
    end_x = x - gfx_canvas.location().x;
    end_y = y - gfx_canvas.location().y;

    g.setColor(gfx_canvas.getForeground());

    // Paint a line if in Draw mode.  No XOR here.
    if (draw_mode == Draw) {
      g.setPaintMode();
      g.drawLine(start_x, start_y, end_x, end_y);
      client.sendGfxMessage(Line, gfx_color.getSelectedIndex(),
			    start_x, start_y, end_x, end_y);
      addFigure(start_x, start_y, end_x, end_y, Line, g.getColor());
      start_x = end_x;
      start_y = end_y;
    }
    // Depending on the type of drawing mode, draw two shapes.
    // The first will XOR over the previously drawn shape, restoring
    // the pixels to their prior state.  The second draws the new
    // shape (which will be erased by the subsequent mouseDrag()).
    else {
      g.setXORMode(gfx_canvas.getBackground());

      switch (draw_mode) {
      case Line:
	drawLine(g, start_x, start_y, prev_x, prev_y);
	drawLine(g, start_x, start_y, end_x, end_y);
	break;
      case Rect:
	drawRect(g, start_x, start_y, prev_x, prev_y);
	drawRect(g, start_x, start_y, end_x, end_y);
	break;
      case FillRect:
	drawRect(g, start_x, start_y, prev_x, prev_y);
	drawRect(g, start_x, start_y, end_x, end_y);
	break;
      case RoundRect:
	drawRoundRect(g, start_x, start_y, prev_x, prev_y);
        drawRoundRect(g, start_x, start_y, end_x, end_y);
	break;
      case FillRoundRect:
	drawRoundRect(g, start_x, start_y, prev_x, prev_y);
        drawRoundRect(g, start_x, start_y, end_x, end_y);
	break;
      case Oval:
	drawOval(g, start_x, start_y, prev_x, prev_y);
        drawOval(g, start_x, start_y, end_x, end_y);
	break;
      case FillOval:
	drawOval(g, start_x, start_y, prev_x, prev_y);
        drawOval(g, start_x, start_y, end_x, end_y);
	break;
      default:
	System.err.println("JavaTalkClientGfx::mouseDrag(): unknown draw mode "
			   + draw_mode);
	break;
      }
    }
    return false;
  }

  // Draw the final shape.
  public boolean mouseUp(Event e, int x, int y)
  {
    if (e.target != gfx_canvas)
      return false;

    Graphics g = gfx_canvas.getGraphics();
      g.setColor(gfx_canvas.getForeground());

    if (draw_mode == Draw)
      ;  // do nothing, already handled by mouseDrag()
    else {
      g.setPaintMode();

      end_x = x - gfx_canvas.location().x;
      end_y = y - gfx_canvas.location().y;

      figures.addElement(new Figure(start_x, start_y, end_x, end_y,
				    draw_mode, current_color));

      client.sendGfxMessage(gfx_mode.getSelectedIndex(),
			    gfx_color.getSelectedIndex(),
			    start_x, start_y, end_x, end_y);

      switch (draw_mode) {
      case Line:
	drawLine(g, start_x, start_y, end_x, end_y);
	break;
      case Rect:
	drawRect(g, start_x, start_y, end_x, end_y);
	break;
      case FillRect:
	drawFillRect(g, start_x, start_y, end_x, end_y);
	break;
      case RoundRect:
	drawRoundRect(g, start_x, start_y, end_x, end_y);
	break;
      case FillRoundRect:
	drawFillRoundRect(g, start_x, start_y, end_x, end_y);
	break;
      case Oval:
	drawOval(g, start_x, start_y, end_x, end_y);
	break;
      case FillOval:
	drawFillOval(g, start_x, start_y, end_x, end_y);
	break;
      default:
	System.err.println("JavaTalkClientGfx::mouseUp(): unknown draw mode "
			   + draw_mode);
	break;
      }
    }
    return true;
  }

  // Interface function for JavaTalkClient when a GFX message is
  // received.
  protected void drawGfx(int draw_mode, int x1, int y1, int x2, int y2,
		      Color color)
  {
    Graphics g = gfx_canvas.getGraphics();

    figures.addElement(new Figure(x1, y1, x2, y2,
				  draw_mode, color));

    g.setColor(color);
      
    switch (draw_mode) {
    case Draw:
    case Line:
      drawLine(g, x1, y1, x2, y2);
      break;
    case Rect:
      drawRect(g, x1, y1, x2, y2);
      break;
    case FillRect:
      drawFillRect(g, x1, y1, x2, y2);
      break;
    case RoundRect:
      drawRoundRect(g, x1, y1, x2, y2);
      break;
    case FillRoundRect:
      drawFillRoundRect(g, x1, y1, x2, y2);
      break;
    case Oval:
      drawOval(g, x1, y1, x2, y2);
      break;
    case FillOval:
      drawFillOval(g, x1, y1, x2, y2);
      break;
    }
  }

  // Stub for Graphics.drawLine.
  private void drawLine(Graphics g, int x1, int y1, int x2, int y2)
  {
    g.drawLine(x1,y1, x2, y2);
  }

  // Stub for Graphics.drawRect, will a little math.
  private void drawRect(Graphics g, int start_x, int start_y,
		       int end_x, int end_y)
  {
    g.drawRect(Math.min(start_x,end_x), Math.min(start_y,end_y),
	       Math.abs(end_x-start_x), Math.abs(end_y-start_y));
  }

  // Stub for Graphics.fillRect, will a little math.
  private void drawFillRect(Graphics g, int start_x, int start_y,
			   int end_x, int end_y)
  {
    g.fillRect(Math.min(start_x,end_x), Math.min(start_y,end_y),
	       Math.abs(end_x-start_x), Math.abs(end_y-start_y));
  }

  // Stub for Graphics.drawRoundRect, will a little math.
  private void drawRoundRect(Graphics g, int start_x, int start_y,
                            int end_x, int end_y)
  {
    g.drawRoundRect(Math.min(start_x,end_x), Math.min(start_y,end_y),
		    Math.abs(end_x-start_x), Math.abs(end_y-start_y),
		    (int)(0.2*Math.abs(end_x-start_x)),
		    (int)(0.2*Math.abs(end_y-start_y)) );
  }

  // Stub for Graphics.fillRoundRect, will a little math.
  private void drawFillRoundRect(Graphics g, int start_x, int start_y,
			           int end_x, int end_y)
  {
    g.fillRoundRect(Math.min(start_x,end_x), Math.min(start_y,end_y),
		    Math.abs(end_x-start_x), Math.abs(end_y-start_y),
		    (int)(0.2*Math.abs(end_x-start_x)),
		    (int)(0.2*Math.abs(end_y-start_y)) );
  }

  // Stub for Graphics.drawOval, will a little math.
  private void drawOval(Graphics g, int start_x, int start_y,
		       int end_x, int end_y)
  {
    g.drawOval(Math.min(start_x,end_x), Math.min(start_y,end_y),
	       Math.abs(end_x-start_x), Math.abs(end_y-start_y));
  }

  // Stub for Graphics.fillOval, will a little math.
  private void drawFillOval(Graphics g, int start_x,int start_y,
			   int end_x,int end_y)
  {
    g.fillOval(Math.min(start_x,end_x), Math.min(start_y,end_y),
	       Math.abs(end_x-start_x), Math.abs(end_y-start_y));
  }

  // Utility method.
  private void addFigure(int start_x, int start_y, int end_x, int end_y,
			int draw_mode, Color color)
  {
    Figure f = new Figure(start_x, start_y, end_x, end_y,
			  draw_mode, current_color);
    figures.addElement(f);
  }
}

/*
 * @author Maharajapuram Balakrishnan.
 * @author Jean-Guy Speton
 * @see    JavaTalkClientGfx
 * 
 * Extension of the java.awt.Canvas so we can override the paint()
 * method.  Otherwise, no frills.
 */
class GfxCanvas extends Canvas {

  private JavaTalkClientGfx parent;

  public GfxCanvas(JavaTalkClientGfx parent)
  {
    this.parent = parent;
  }

  public void paint(Graphics g) 
  {
    parent.paint(g);
  }
}

/*
 * @author Maharajapuram Balakrishnan
 * @author Jean-Guy Speton
 * @see    JavaTalkClientGfx
 *
 * Simple structure to maintain graphic element attributes in our Vector
 * of such elements.
 */
class Figure extends Object {
  public int start_x, start_y, end_x, end_y;
  public int draw_mode;
  public Color color;

  public Figure(int start_x, int start_y, int end_x, int end_y,
                int draw_mode, Color color)
  {
    this.start_x = start_x;
    this.start_y = start_y;
    this.end_x   = end_x;
    this.end_y   = end_y;
    this.draw_mode = draw_mode;
    this.color = color;
  }

  public String toString()
  {
    return "[x1=" + start_x + ",y1=" + start_y + ",x2=" + end_x +
      ",y2=" + end_y + ",draw=" + draw_mode + ",color=" + color + "]";
  }

}
