/**
 * @author M. Balakrishnan
 * @author J. speton
 * @see JavaTalkClient
 * ChatArea window
 */

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;

public class JavaTalkClientTxt extends Frame {

  private JavaTalkClient client;
  private GridBagLayout text_layout;
  protected TextArea text_area;
  private TextField text_field;
  private String user_name;
  
  public JavaTalkClientTxt (JavaTalkClient client, String user_name)
  {
    super("JavaTalk Text Window");

    this.client = client;
    this.user_name = user_name;
    text_layout = new GridBagLayout();
    this.setLayout(text_layout);
    text_area = new TextArea("");
    text_area.setEditable(false);
    constrain(this, text_area, 0, 0, 2, 1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER, 1.0, 1.0, 2, 2, 2, 2);
    constrain(this, new Label(user_name + ":"), 0, 1, 1, 1, 
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 0.0, 0.0, 2, 2, 2, 0);
    text_field = new TextField("");
    constrain(this, text_field, 1, 1, 1, 1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER, 1.0, 0.0, 2, 0, 2, 2);
  }

  public boolean action(Event event, Object arg)
  {
    if (event.target == text_field) {
      if (text_field.getText().length() > 0) {
	String selected = client.user_list.getSelectedItem();
	// System.err.println("selected: " + selected);

	// broadcast to all if no target selected
	if (selected == null) {
	  synchronized (client.out) {
	    try {
	      client.out.writeInt(JavaTalkMessage.TEXT);
	      client.out.writeBytes(client.name.getText() + "\n");
	      client.out.writeBytes(text_field.getText() + "\n");
	      text_field.setText("");
	    }
	    catch (IOException e)
	      System.err.println("Text: " + e);
	  }
	}
	else {   // send private message to selected user
	  synchronized (client.out) {
	    try {
	      client.out.writeInt(JavaTalkMessage.PRIV_TEXT);
	      client.out.writeBytes(selected + "\n");
	      client.out.writeBytes(client.name.getText() + "\n");
	      client.out.writeBytes(text_field.getText() + "\n");
	      text_area.appendText("--> " + selected + ": " +
				   text_field.getText() + "\n");
	      text_field.setText("");
	    }
	    catch (IOException e)
	      System.err.println("Private text: " + e);
	  }
	}
      }
      return true;
    }
    return false;
  }

  // Simplifies using GridBagContraints objects.
  // Mercilessly lifted from _Java in a Nutshell_.
  public void constrain(Container container, Component component,
			int grid_x, int grid_y,
			int grid_width, int grid_height,
			int fill, int anchor, double weight_x, double weight_y,
			int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);

    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }

  public void constrain(Container container, Component component,
			int grid_x, int grid_y, 
			int grid_width, int grid_height)
    {
      constrain(container, component, grid_x, grid_y,
		grid_width, grid_height, GridBagConstraints.BOTH,
		GridBagConstraints.CENTER, 0.0, 0.0, 2, 2, 2, 2);
    }



}
